@@ -8,10 +8,12 @@ if __name__ == '__main__':
 import subprocess
 import os
 import sys
+import py
 import pytest
 import stat
 import shutil
 import filecmp
+import tempfile
 import time
 import errno
 import sys
@@ -56,13 +58,29 @@ def invoke_mount_fuse_drop_privileges(mnt_dir, name, options):
 
     return invoke_mount_fuse(mnt_dir, name, options + ('drop_privileges',))
 
+class raii_tmpdir:
+    def __init__(self):
+        self.d = tempfile.mkdtemp()
+
+    def __str__(self):
+        return str(self.d)
+
+    def mkdir(self, path):
+        return py.path.local(str(self.d)).mkdir(path)
+
+@pytest.fixture
+def short_tmpdir():
+    return raii_tmpdir()
+
 @pytest.mark.parametrize("cmdline_builder", (invoke_directly, invoke_mount_fuse,
                                              invoke_mount_fuse_drop_privileges))
 @pytest.mark.parametrize("options", powerset(options))
 @pytest.mark.parametrize("name", ('hello', 'hello_ll'))
-def test_hello(tmpdir, name, options, cmdline_builder):
+def test_hello(tmpdir, name, options, cmdline_builder, output_checker):
     mnt_dir = str(tmpdir)
-    mount_process = subprocess.Popen(cmdline_builder(mnt_dir, name, options))
+    mount_process = subprocess.Popen(
+        cmdline_builder(mnt_dir, name, options),
+        stdout=output_checker.fd, stderr=output_checker.fd)
     try:
         wait_for_mount(mount_process, mnt_dir)
         assert os.listdir(mnt_dir) == [ 'hello' ]
@@ -84,19 +102,18 @@ def test_hello(tmpdir, name, options, cmdline_builder):
 @pytest.mark.parametrize("writeback", (False, True))
 @pytest.mark.parametrize("name", ('passthrough', 'passthrough_fh', 'passthrough_ll'))
 @pytest.mark.parametrize("debug", (False, True))
-def test_passthrough(tmpdir, name, debug, capfd, writeback):
-    
+def test_passthrough(short_tmpdir, name, debug, output_checker, writeback):
     # Avoid false positives from libfuse debug messages
     if debug:
-        capfd.register_output(r'^   unique: [0-9]+, error: -[0-9]+ .+$',
-                              count=0)
+        output_checker.register_output(r'^   unique: [0-9]+, error: -[0-9]+ .+$',
+                                       count=0)
 
     # test_syscalls prints "No error" under FreeBSD
-    capfd.register_output(r"^ \d\d \[[^\]]+ message: 'No error: 0'\]",
-                          count=0)
+    output_checker.register_output(r"^ \d\d \[[^\]]+ message: 'No error: 0'\]",
+                                   count=0)
 
-    mnt_dir = str(tmpdir.mkdir('mnt'))
-    src_dir = str(tmpdir.mkdir('src'))
+    mnt_dir = str(short_tmpdir.mkdir('mnt'))
+    src_dir = str(short_tmpdir.mkdir('src'))
 
     cmdline = base_cmdline + \
               [ pjoin(basename, 'example', name),
@@ -110,7 +127,8 @@ def test_passthrough(tmpdir, name, debug, capfd, writeback):
         cmdline.append('-o')
         cmdline.append('writeback')
         
-    mount_process = subprocess.Popen(cmdline)
+    mount_process = subprocess.Popen(cmdline, stdout=output_checker.fd,
+                                     stderr=output_checker.fd)
     try:
         wait_for_mount(mount_process, mnt_dir)
         work_dir = mnt_dir + src_dir
@@ -145,7 +163,7 @@ def test_passthrough(tmpdir, name, debug, capfd, writeback):
             # When writeback caching is enabled, kernel has to open files for
             # reading even when userspace opens with O_WDONLY. This fails if the
             # filesystem process doesn't have special permission.
-            syscall_test_cmd.append('-52')
+            syscall_test_cmd.append('-53')
         subprocess.check_call(syscall_test_cmd)
     except:
         cleanup(mount_process, mnt_dir)
@@ -153,9 +171,65 @@ def test_passthrough(tmpdir, name, debug, capfd, writeback):
     else:
         umount(mount_process, mnt_dir)
 
+@pytest.mark.parametrize("cache", (False, True))
+def test_passthrough_hp(short_tmpdir, cache, output_checker):
+    mnt_dir = str(short_tmpdir.mkdir('mnt'))
+    src_dir = str(short_tmpdir.mkdir('src'))
+
+    cmdline = base_cmdline + \
+              [ pjoin(basename, 'example', 'passthrough_hp'),
+                src_dir, mnt_dir ]
+
+    if not cache:
+        cmdline.append('--nocache')
+        
+    mount_process = subprocess.Popen(cmdline, stdout=output_checker.fd,
+                                     stderr=output_checker.fd)
+    try:
+        wait_for_mount(mount_process, mnt_dir)
+
+        tst_statvfs(mnt_dir)
+        tst_readdir(src_dir, mnt_dir)
+        tst_readdir_big(src_dir, mnt_dir)
+        tst_open_read(src_dir, mnt_dir)
+        tst_open_write(src_dir, mnt_dir)
+        tst_create(mnt_dir)
+        if not cache:
+            tst_passthrough(src_dir, mnt_dir)
+        tst_append(src_dir, mnt_dir)
+        tst_seek(src_dir, mnt_dir)
+        tst_mkdir(mnt_dir)
+        tst_rmdir(src_dir, mnt_dir)
+        tst_unlink(src_dir, mnt_dir)
+        tst_symlink(mnt_dir)
+        if os.getuid() == 0:
+            tst_chown(mnt_dir)
+
+        # Underlying fs may not have full nanosecond resolution
+        tst_utimens(mnt_dir, ns_tol=1000)
+
+        tst_link(mnt_dir)
+        tst_truncate_path(mnt_dir)
+        tst_truncate_fd(mnt_dir)
+        tst_open_unlink(mnt_dir)
+
+        # test_syscalls assumes that changes in source directory
+        # will be reflected immediately in mountpoint, so we
+        # can't use it.
+        if not cache:
+            syscall_test_cmd = [ os.path.join(basename, 'test', 'test_syscalls'),
+                             mnt_dir, ':' + src_dir ]
+            subprocess.check_call(syscall_test_cmd)
+    except:
+        cleanup(mount_process, mnt_dir)
+        raise
+    else:
+        umount(mount_process, mnt_dir)
+
+        
 @pytest.mark.skipif(fuse_proto < (7,11),
                     reason='not supported by running kernel')
-def test_ioctl(tmpdir):
+def test_ioctl(tmpdir, output_checker):
     progname = pjoin(basename, 'example', 'ioctl')
     if not os.path.exists(progname):
         pytest.skip('%s not built' % os.path.basename(progname))
@@ -163,7 +237,8 @@ def test_ioctl(tmpdir):
     mnt_dir = str(tmpdir)
     testfile = pjoin(mnt_dir, 'fioc')
     cmdline = base_cmdline + [progname, '-f', mnt_dir ]
-    mount_process = subprocess.Popen(cmdline)
+    mount_process = subprocess.Popen(cmdline, stdout=output_checker.fd,
+                                     stderr=output_checker.fd)
     try:
         wait_for_mount(mount_process, mnt_dir)
 
@@ -183,11 +258,12 @@ def test_ioctl(tmpdir):
     else:
         umount(mount_process, mnt_dir)
 
-def test_poll(tmpdir):
+def test_poll(tmpdir, output_checker):
     mnt_dir = str(tmpdir)
     cmdline = base_cmdline + [pjoin(basename, 'example', 'poll'),
                '-f', mnt_dir ]
-    mount_process = subprocess.Popen(cmdline)
+    mount_process = subprocess.Popen(cmdline, stdout=output_checker.fd,
+                                     stderr=output_checker.fd)
     try:
         wait_for_mount(mount_process, mnt_dir)
         cmdline = base_cmdline + \
@@ -199,7 +275,7 @@ def test_poll(tmpdir):
     else:
         umount(mount_process, mnt_dir)
 
-def test_null(tmpdir):
+def test_null(tmpdir, output_checker):
     progname = pjoin(basename, 'example', 'null')
     if not os.path.exists(progname):
         pytest.skip('%s not built' % os.path.basename(progname))
@@ -208,7 +284,8 @@ def test_null(tmpdir):
     with open(mnt_file, 'w') as fh:
         fh.write('dummy')
     cmdline = base_cmdline + [ progname, '-f', mnt_file ]
-    mount_process = subprocess.Popen(cmdline)
+    mount_process = subprocess.Popen(cmdline, stdout=output_checker.fd,
+                                     stderr=output_checker.fd)
     def test_fn(name):
         return os.stat(name).st_size > 4000
     try:
@@ -227,7 +304,7 @@ def test_null(tmpdir):
 @pytest.mark.skipif(fuse_proto < (7,12),
                     reason='not supported by running kernel')
 @pytest.mark.parametrize("notify", (True, False))
-def test_notify_inval_entry(tmpdir, notify):
+def test_notify_inval_entry(tmpdir, notify, output_checker):
     mnt_dir = str(tmpdir)
     cmdline = base_cmdline + \
               [ pjoin(basename, 'example', 'notify_inval_entry'),
@@ -235,7 +312,8 @@ def test_notify_inval_entry(tmpdir, notify):
                 '--timeout=5', mnt_dir ]
     if not notify:
         cmdline.append('--no-notify')
-    mount_process = subprocess.Popen(cmdline)
+    mount_process = subprocess.Popen(cmdline, stdout=output_checker.fd,
+                                     stderr=output_checker.fd)
     try:
         wait_for_mount(mount_process, mnt_dir)
         fname = pjoin(mnt_dir, os.listdir(mnt_dir)[0])
@@ -261,19 +339,20 @@ def test_notify_inval_entry(tmpdir, notify):
 
 @pytest.mark.skipif(os.getuid() != 0,
                     reason='needs to run as root')
-def test_cuse(capfd):
+def test_cuse(output_checker):
 
     # Valgrind warns about unknown ioctls, that's ok
-    capfd.register_output(r'^==([0-9]+).+unhandled ioctl.+\n'
-                          r'==\1== \s{3}.+\n'
-                          r'==\1== \s{3}.+$', count=0)
+    output_checker.register_output(r'^==([0-9]+).+unhandled ioctl.+\n'
+                                   r'==\1== \s{3}.+\n'
+                                   r'==\1== \s{3}.+$', count=0)
 
     devname = 'cuse-test-%d' % os.getpid()
     devpath = '/dev/%s' % devname
     cmdline = base_cmdline + \
               [ pjoin(basename, 'example', 'cuse'),
                 '-f', '--name=%s' % devname ]
-    mount_process = subprocess.Popen(cmdline)
+    mount_process = subprocess.Popen(cmdline, stdout=output_checker.fd,
+                                     stderr=output_checker.fd)
 
     cmdline = base_cmdline + \
               [ pjoin(basename, 'example', 'cuse_client'),
