@@ -11,21 +11,28 @@
 #include <utime.h>
 #include <errno.h>
 #include <assert.h>
+#include <sys/socket.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <sys/un.h>
+
+#ifndef ALLPERMS
+# define ALLPERMS (S_ISUID|S_ISGID|S_ISVTX|S_IRWXU|S_IRWXG|S_IRWXO)/* 07777 */
+#endif
 
 
 static char testfile[1024];
 static char testfile2[1024];
 static char testdir[1024];
 static char testdir2[1024];
-static char subfile[1024];
+static char testsock[1024];
+static char subfile[1280];
 
 static char testfile_r[1024];
 static char testfile2_r[1024];
 static char testdir_r[1024];
 static char testdir2_r[1024];
-static char subfile_r[1024];
+static char subfile_r[1280];
 
 static char testname[256];
 static char testdata[] = "abcdefghijklmnopqrstuvwxyz";
@@ -63,6 +70,11 @@ static void test_error(const char *func, const char *msg, ...)
 	fprintf(stderr, "\n");
 }
 
+static int is_dot_or_dotdot(const char *name) {
+    return name[0] == '.' &&
+           (name[1] == '\0' || (name[1] == '.' && name[2] == '\0'));
+}
+
 static void success(void)
 {
 	fprintf(stderr, "%s OK\n", testname);
@@ -163,8 +175,9 @@ static int check_mode(const char *path, mode_t mode)
 		PERROR("lstat");
 		return -1;
 	}
-	if ((stbuf.st_mode & 07777) != mode) {
-		ERROR("mode 0%o instead of 0%o", stbuf.st_mode & 07777, mode);
+	if ((stbuf.st_mode & ALLPERMS) != mode) {
+		ERROR("mode 0%o instead of 0%o", stbuf.st_mode & ALLPERMS,
+		      mode);
 		return -1;
 	}
 	return 0;
@@ -178,8 +191,9 @@ static int fcheck_mode(int fd, mode_t mode)
 		PERROR("fstat");
 		return -1;
 	}
-	if ((stbuf.st_mode & 07777) != mode) {
-		ERROR("mode 0%o instead of 0%o", stbuf.st_mode & 07777, mode);
+	if ((stbuf.st_mode & ALLPERMS) != mode) {
+		ERROR("mode 0%o instead of 0%o", stbuf.st_mode & ALLPERMS,
+		      mode);
 		return -1;
 	}
 	return 0;
@@ -375,10 +389,6 @@ static int check_dir_contents(const char *path, const char **contents)
 		found[i] = 0;
 		cont[i] = contents[i];
 	}
-	found[i] = 0;
-	cont[i++] = ".";
-	found[i] = 0;
-	cont[i++] = "..";
 	cont[i] = NULL;
 
 	dp = opendir(path);
@@ -399,6 +409,8 @@ static int check_dir_contents(const char *path, const char **contents)
 			}
 			break;
 		}
+		if (is_dot_or_dotdot(de->d_name))
+			continue;
 		for (i = 0; cont[i] != NULL; i++) {
 			assert(i < MAX_ENTRIES);
 			if (strcmp(cont[i], de->d_name) == 0) {
@@ -491,7 +503,7 @@ static int cleanup_dir(const char *path, const char **dir_files, int quiet)
 
 	for (i = 0; dir_files[i]; i++) {
 		int res;
-		char fpath[1024];
+		char fpath[1280];
 		sprintf(fpath, "%s/%s", path, dir_files[i]);
 		res = unlink(fpath);
 		if (res == -1 && !quiet) {
@@ -524,7 +536,7 @@ static int create_dir(const char *path, const char **dir_files)
 		return -1;
 
 	for (i = 0; dir_files[i]; i++) {
-		char fpath[1024];
+		char fpath[1280];
 		sprintf(fpath, "%s/%s", path, dir_files[i]);
 		res = create_file(fpath, "", 0);
 		if (res == -1) {
@@ -1040,7 +1052,7 @@ static int do_test_open(int exist, int flags, const char *flags_str, int mode)
 		err += check_mode(testfile, mode);
 	err += check_nlink(testfile, 1);
 	err += check_size(testfile, currlen);
-	if (exist && !(flags & O_TRUNC) && (mode & 0400))
+	if (exist && !(flags & O_TRUNC) && (mode & S_IRUSR))
 		err += check_data(testfile, testdata2, 0, testdata2len);
 
 	res = write(fd, data, datalen);
@@ -1057,7 +1069,7 @@ static int do_test_open(int exist, int flags, const char *flags_str, int mode)
 
 			err += check_size(testfile, currlen);
 
-			if (mode & 0400) {
+			if (mode & S_IRUSR) {
 				err += check_data(testfile, data, 0, datalen);
 				if (exist && !(flags & O_TRUNC) &&
 				    testdata2len > datalen)
@@ -1445,7 +1457,7 @@ static int test_rename_dir_loop(void)
 #define PATH(p)		(snprintf(path, sizeof path, "%s/%s", testdir, p), path)
 #define PATH2(p)	(snprintf(path2, sizeof path2, "%s/%s", testdir, p), path2)
 
-	char path[1024], path2[1024];
+	char path[1280], path2[1280];
 	int err = 0;
 	int res;
 
@@ -1725,6 +1737,54 @@ static int test_mkdir(void)
 	return 0;
 }
 
+static int test_socket(void)
+{
+	struct sockaddr_un su;
+	int fd;
+	int res;
+	int err = 0;
+
+	start_test("socket");
+	if (strlen(testsock) + 1 > sizeof(su.sun_path)) {
+		fprintf(stderr, "Need to shorten mount point by %lu chars\n",
+			strlen(testsock) + 1 - sizeof(su.sun_path));
+		return -1;
+	}
+	unlink(testsock);
+	fd = socket(AF_UNIX, SOCK_STREAM, 0);
+	if (fd < 0) {
+		PERROR("socket");
+		return -1;
+	}
+	su.sun_family = AF_UNIX;
+	strncpy(su.sun_path, testsock, sizeof(su.sun_path) - 1);
+	su.sun_path[sizeof(su.sun_path) - 1] = '\0';
+	res = bind(fd, (struct sockaddr*)&su, sizeof(su));
+	if (res == -1) {
+		PERROR("bind");
+		return -1;
+	}
+
+	res = check_type(testsock, S_IFSOCK);
+	if (res == -1)
+		return -1;
+	err += check_nlink(testsock, 1);
+	close(fd);
+	res = unlink(testsock);
+	if (res == -1) {
+		PERROR("unlink");
+		return -1;
+	}
+	res = check_nonexist(testsock);
+	if (res == -1)
+		return -1;
+	if (err)
+		return -1;
+
+	success();
+	return 0;
+}
+
 #define test_create_ro_dir(flags)	 \
 	do_test_create_ro_dir(flags, #flags)
 
@@ -1813,6 +1873,7 @@ int main(int argc, char *argv[])
 	sprintf(testdir, "%s/testdir", basepath);
 	sprintf(testdir2, "%s/testdir2", basepath);
 	sprintf(subfile, "%s/subfile", testdir2);
+	sprintf(testsock, "%s/testsock", basepath);
 
 	sprintf(testfile_r, "%s/testfile", realpath);
 	sprintf(testfile2_r, "%s/testfile2", realpath);
@@ -1836,6 +1897,7 @@ int main(int argc, char *argv[])
 	err += test_rename_dir();
 	err += test_rename_dir_loop();
 	err += test_seekdir();
+	err += test_socket();
 	err += test_utime();
 	err += test_truncate(0);
 	err += test_truncate(testdatalen / 2);
@@ -1894,6 +1956,7 @@ int main(int argc, char *argv[])
 
 	unlink(testfile);
 	unlink(testfile2);
+	unlink(testsock);
 	rmdir(testdir);
 	rmdir(testdir2);
 
